;--------------------------------------------------------------------------------
; SCROLLING
; Created by Matthew Smith
;--------------------------------------------------------------------------------
; Maps are designed for 512x17 in size
;  - width can be customised (recommended by 256 incrementsdue to math)
;  - height can be customised (adding or removing MAPSCREENBUFFERROWX entries)
;  - changes will need to be made to the shift/copy routines to accommodate
; 
; Buffer is designed for 42x17 in size and scrolling from right to left
; Columns 41/42 are used to load the next incoming characters
; Scrolling based on 8 pixels (or 2 characters) for better performance
;
; Process:
; - Draw screen background (a left and right side) and save screen
; - Shift background using the 'Under the hood' (UTH) method
; - At completion of the shift, restore the background zone(s) to their starting 
;   position, shift the buffer columns over 2 characters and copy in the next
;   2 characters
;
; Notes:
; - As the UTH method is used we don't need to continually draw the background
; = we need to add a 'drawwait' call to ensure the restore/shift/copy routine
;   doesn't tear the screen but this introduces a 1 frame delay (and potentially
;   more). I've added a counter to simulate this during fine scroll
; - Overall the scroll shifts every 2nd (or could be more if required) frame
;   then allowing time for other updates on the other frame(s)
; - On PAL there is frame variations (TBD) making the scrolling less smooth
; - If moving 8 pixels just call CourseScroll routine directly
; - The routine could be changed to copy the map data out of ROM but larger maps 
;   might be difficult to access due to bank requirements
;--------------------------------------------------------------------------------
; BUFFER
;--------------------------------------------------------------------------------
MAPSCREENBUFFERROWA     EQU MAPSCREENBUFFERRAM
MAPSCREENBUFFERROWB     EQU MAPSCREENBUFFERROWA+42
MAPSCREENBUFFERROWC     EQU MAPSCREENBUFFERROWB+42
MAPSCREENBUFFERROWD     EQU MAPSCREENBUFFERROWC+42
MAPSCREENBUFFERROWE     EQU MAPSCREENBUFFERROWD+42
MAPSCREENBUFFERROWF     EQU MAPSCREENBUFFERROWE+42
MAPSCREENBUFFERROWG     EQU MAPSCREENBUFFERROWF+42
MAPSCREENBUFFERROWH     EQU MAPSCREENBUFFERROWG+42
MAPSCREENBUFFERROWI     EQU MAPSCREENBUFFERROWH+42
MAPSCREENBUFFERROWJ     EQU MAPSCREENBUFFERROWI+42
MAPSCREENBUFFERROWK     EQU MAPSCREENBUFFERROWJ+42
MAPSCREENBUFFERROWL     EQU MAPSCREENBUFFERROWK+42
MAPSCREENBUFFERROWM     EQU MAPSCREENBUFFERROWL+42
MAPSCREENBUFFERROWN     EQU MAPSCREENBUFFERROWM+42
MAPSCREENBUFFERROWO     EQU MAPSCREENBUFFERROWN+42
MAPSCREENBUFFERROWP     EQU MAPSCREENBUFFERROWO+42
MAPSCREENBUFFERROWQ     EQU MAPSCREENBUFFERROWP+42
;--------------------------------------------------------------------------------
; VARIABLES
;--------------------------------------------------------------------------------
MAPSCROLLENABLED        EQU MAPSCREENVARS
MAPSHIFTINDEX           EQU MAPSCROLLENABLED+1
MAPSHIFTINDEXEXIT       EQU MAPSHIFTINDEX+1
MAPSCROLLSPEED          EQU MAPSHIFTINDEXEXIT+1
MAPLOOPENABLED          EQU MAPSCROLLSPEED+1
MAPUPDATECOUNTER        EQU MAPLOOPENABLED+1
;--------------------------------------------------------------------------------
; SETUP
;--------------------------------------------------------------------------------
.InitialiseMapScreenConfiguration
InitialiseMapScreenConfiguration
  lda #0
  sta MAPSHIFTINDEX
  lda #1
  sta MAPSCROLLENABLED
  sta MAPLOOPENABLED
  lda #1
  sta MAPSCROLLSPEED
  ;depending on the scrolling set the following for exit of shift
  ;1 pixel-7, 2 pixels-6, 4 pixels-4
  cmp #1
  bne InitialiseMapScreenConfigurationCheckShiftIndexTwo
  lda #7
  jmp InitialiseMapScreenConfigurationFinaliseShiftIndexExit
InitialiseMapScreenConfigurationCheckShiftIndexTwo
  cmp #2
  bne InitialiseMapScreenConfigurationCheckShiftIndexFour
  lda #6     
  jmp InitialiseMapScreenConfigurationFinaliseShiftIndexExit
InitialiseMapScreenConfigurationCheckShiftIndexFour
  lda #4
InitialiseMapScreenConfigurationFinaliseShiftIndexExit
  sta MAPSHIFTINDEXEXIT
  rts
;--------------------------------------------------------------------------------
; ROUTINES
;--------------------------------------------------------------------------------

;If you want to scroll < 8 pixels then call this routine
.MapScreenScrollLeftFineScroll
MapScreenScrollLeftFineScroll
  ;scroll enabled?
  lda MAPSCROLLENABLED
  beq MapScreenScrollLeftFineScrollLoopExit
  ;end of shift?
  ;NOTE: we can cheat and skip the last shift as it will be "done" during final course scroll...
  lda MAPSHIFTINDEX
  cmp MAPSHIFTINDEXEXIT
  bcs MapScreenScrollLeftCourseScroll
  ;increment map update counter
  inc MAPUPDATECOUNTER
  ;process?
  ;NOTE: as drawait (in course scroll) skips a frame check the framecounter to do the same for fine scrolling 
  ;      to smooth the overall process
	lda MAPUPDATECOUNTER
	and #$01
	cmp #0
  bne MapScreenScrollLeftFineScrollLoopExit
  ;set speed
  lda MAPSCROLLSPEED
  sta temp3
  ;set row
  lda #5
  sta temp4
MapScreenScrollLeftFineScrollLoop
  ldy temp4
  cpy #22
  bcs MapScreenScrollLeftFineScrollLoopFinalise
  ;col 0 (left side of screen)
  lda #0
  jsr ShiftBlockLeft
  ;col 1 (right side of screen)
  ldy temp4
  lda #1
  jsr ShiftBlockLeft
  ;increment row
  inc temp4
  jmp MapScreenScrollLeftFineScrollLoop
MapScreenScrollLeftFineScrollLoopFinalise
  ;increment shiftindex
  clc
  lda MAPSHIFTINDEX
  adc MAPSCROLLSPEED
  sta MAPSHIFTINDEX
MapScreenScrollLeftFineScrollLoopExit
  rts

;If you want to scroll > 4 pixels (8 only) then call this routine
.MapScreenScrollLeftCourseScroll
MapScreenScrollLeftCourseScroll
  ;delay 7800basic drawing (so we can shift the entire contents)
  ;NOTE: this puts a one frame delay into the mix
  jsr drawwait
  ;reset shiftindex
  lda #0
  sta MAPSHIFTINDEX
  ;update screen
  ;NOTE: these are set inline to save a few cycles but could be 
  ;restored into individual calls if required
  ;jsr ResetMapScreenLeftPosition
  ;jsr ShiftMapLeftScreenBufferColumnsAll
  ;jsr CopyStorageToMapScreen
  ;jsr HasReachedEndOfMap
  ;rts

ResetMapScreenLeftPosition
  ;row
  lda #5
  sta temp4
ResetMapScreenLeftPositionLoop
  ldy temp4
  cpy #22
  bcs ResetMapScreenLeftPositionLoopExit
  ;col 0 (left side of screen)
  lda #0
  sta temp3
  lda #0
  jsr ShiftBlockTo
  ;col 1 (right side of screen)
  lda #84
  sta temp3
  ldy temp4
  lda #1
  jsr ShiftBlockTo
  ;increment row
  inc temp4
  jmp ResetMapScreenLeftPositionLoop
ResetMapScreenLeftPositionLoopExit
  ;rts

ShiftMapLeftScreenBufferColumnsAll
  ldy #2
  ldx #0
ShiftMapLeftScreenBufferColumnsAllLoop
  cpy #42
  bcs ShiftMapLeftScreenBufferColumnsAllExit
  jsr ShiftMapScreenBufferColumnDo
  inx 
  iny
  jmp ShiftMapLeftScreenBufferColumnsAllLoop
ShiftMapLeftScreenBufferColumnsAllExit
  ;rts

CopyStorageToMapScreen
  jsr CopyMapStorageToMapScreenBufferRightColumns
  ;Increment mapscreen right edge pointer
  lda #2
  jsr IncrementMapStorageRightEdgePointerBy
  ;rts

HasReachedEndOfMap
  ;Validate loop map?
  ldx #$01
  lda mapStorageRightEdgePointer,x
  cmp #$41    ;TODO: convert to settable value
  bne HasReachedEndOfMapExit
  lda #$FD    ;TODO: convert to settable value
  cmp mapStorageRightEdgePointer
  bcs HasReachedEndOfMapExit
  ;stop or loop?
  lda MAPLOOPENABLED
  bne HasReachedEndOfMapDoLoop
  ;Stop
  lda #$00
  sta MAPSCROLLENABLED
  jmp HasReachedEndOfMapExit
HasReachedEndOfMapDoLoop
  ;Loop (reset map to start)
  lda #MAPSTORAGERAMLO
  sta mapStorageRightEdgePointer
  lda #MAPSTORAGERAMHI
  ldx #$01
  sta mapStorageRightEdgePointer,x
HasReachedEndOfMapExit
  rts

ShiftMapScreenBufferColumnDo
  lda MAPSCREENBUFFERROWA,y
  sta MAPSCREENBUFFERROWA,x
  lda MAPSCREENBUFFERROWB,y
  sta MAPSCREENBUFFERROWB,x
  lda MAPSCREENBUFFERROWC,y
  sta MAPSCREENBUFFERROWC,x
  lda MAPSCREENBUFFERROWD,y
  sta MAPSCREENBUFFERROWD,x
  lda MAPSCREENBUFFERROWE,y
  sta MAPSCREENBUFFERROWE,x
  lda MAPSCREENBUFFERROWF,y
  sta MAPSCREENBUFFERROWF,x
  lda MAPSCREENBUFFERROWG,y
  sta MAPSCREENBUFFERROWG,x
  lda MAPSCREENBUFFERROWH,y
  sta MAPSCREENBUFFERROWH,x
  lda MAPSCREENBUFFERROWI,y
  sta MAPSCREENBUFFERROWI,x
  lda MAPSCREENBUFFERROWJ,y
  sta MAPSCREENBUFFERROWJ,x
  lda MAPSCREENBUFFERROWK,y
  sta MAPSCREENBUFFERROWK,x
  lda MAPSCREENBUFFERROWL,y
  sta MAPSCREENBUFFERROWL,x
  lda MAPSCREENBUFFERROWM,y
  sta MAPSCREENBUFFERROWM,x
  lda MAPSCREENBUFFERROWN,y
  sta MAPSCREENBUFFERROWN,x
  lda MAPSCREENBUFFERROWO,y
  sta MAPSCREENBUFFERROWO,x
  lda MAPSCREENBUFFERROWP,y
  sta MAPSCREENBUFFERROWP,x
  lda MAPSCREENBUFFERROWQ,y
  sta MAPSCREENBUFFERROWQ,x
  rts

.CopyMapStorageToMapScreenBufferRightColumns
CopyMapStorageToMapScreenBufferRightColumns
  ldy #0
  ldx #40
CopyMapStorageToMapScreenBufferRightColumnsLoop
  cpy #2
  bcs CopyMapStorageToMapScreenBufferRightColumnsExit
  jsr CopyMapStorageColumnToMapScreenBufferColumnDo
  iny
  inx
  jmp CopyMapStorageToMapScreenBufferRightColumnsLoop
CopyMapStorageToMapScreenBufferRightColumnsExit
  rts

CopyMapStorageColumnToMapScreenBufferColumnDo
  lda mapStorageRightEdgePointer
  sta pointer
  lda mapStorageRightEdgePointer+1
  sta pointer+1 
  lda (pointer),y
  sta MAPSCREENBUFFERROWA,x
  inc pointer+1
  inc pointer+1
  lda (pointer),y
  sta MAPSCREENBUFFERROWB,x
  inc pointer+1
  inc pointer+1
  lda (pointer),y
  sta MAPSCREENBUFFERROWC,x
  inc pointer+1
  inc pointer+1
  lda (pointer),y
  sta MAPSCREENBUFFERROWD,x
  inc pointer+1
  inc pointer+1
  lda (pointer),y
  sta MAPSCREENBUFFERROWE,x
  inc pointer+1
  inc pointer+1
  lda (pointer),y
  sta MAPSCREENBUFFERROWF,x
  inc pointer+1
  inc pointer+1
  lda (pointer),y
  sta MAPSCREENBUFFERROWG,x
  inc pointer+1
  inc pointer+1
  lda (pointer),y
  sta MAPSCREENBUFFERROWH,x
  inc pointer+1
  inc pointer+1
  lda (pointer),y
  sta MAPSCREENBUFFERROWI,x
  inc pointer+1
  inc pointer+1
  lda (pointer),y
  sta MAPSCREENBUFFERROWJ,x
  inc pointer+1
  inc pointer+1
  lda (pointer),y
  sta MAPSCREENBUFFERROWK,x
  inc pointer+1
  inc pointer+1
  lda (pointer),y
  sta MAPSCREENBUFFERROWL,x
  inc pointer+1
  inc pointer+1
  lda (pointer),y
  sta MAPSCREENBUFFERROWM,x
  inc pointer+1
  inc pointer+1
  lda (pointer),y
  sta MAPSCREENBUFFERROWN,x
  inc pointer+1
  inc pointer+1
  lda (pointer),y
  sta MAPSCREENBUFFERROWO,x
  inc pointer+1
  inc pointer+1
  lda (pointer),y
  sta MAPSCREENBUFFERROWP,x
  inc pointer+1
  inc pointer+1
  lda (pointer),y
  sta MAPSCREENBUFFERROWQ,x
  rts